/**************************************************************************************************
* Project name:
*    Light to Frequency
* Copyright:
*    (c) MikroElektronika, 2011.
* Revision History:
*    20111227:
*      - initial release (M.V. & JK);
*    20120418:
*      - modified for EasyPIC PRO v7 (JK);
*    20130612:
*      - modified for EasyAVR v7 (MV);
* Description:
*    This project demonstrates working with TSL230RD Light-to-frequency
*    converter on LightHz Click board. Pulses from TSL230BR are counted using
*    PD2/INT0 interrupt on rising edge on pin PD2 of ATmega32 MCU.
*    Timer1 generates interrupt after 1/8 second for calculating frequency.
*    Result is displayed in linear scale on PORTC diodes, and sent using
*    UART communication.
*
* * Test configuration:
*    MCU:             ATmega32
*                     http://www.atmel.com/Images/doc2503.pdf
*    Dev.Board:       EasyAVR7
*                     ac:EasyAVR7
*                     http://www.mikroe.com/easyavr/
*    Oscillator:      Oscillator Frequency 8.0000 MHz, 8.0000 MHz Crystal
*    ext. modules:    LightHz click
*                     ac:LightHz_Click
*                     http://www.mikroe.com/click/lighthz/
*    SW:              mikroC PRO for AVR
*                     http://www.mikroe.com/mikroc/avr/
*
* NOTES:
*    - Place LightHz Click to microBUS socket 1
*    - Turn on LEDs on PORTC, SW10.7
*    - Turn on USB UART Rx and Tx to PD0 and PD1 (SW10.1 and SW10.2) and choose
       RS232/USB UART via jumpers J12 and J23
**************************************************************************************************/

/**************************************************************************************************
* DGlobal variables
**************************************************************************************************/
char DIODES at PORTC;
char DIODES_direction at DDRC;
sbit OUT_direction  at DDD2_bit;

unsigned long cnt, freq, old_freq;
char txt[12];
int timer_cnt,freq_read_done = 0;

/**************************************************************************************************
* Interrupt routine
**************************************************************************************************/
void interrupt_INT0() iv IVT_ADDR_INT0 ics ICS_AUTO {
    INTF0_bit = 0;
    cnt++;                             // Count interrupts on PD2/INT pin
}

void interrupt_TMR1_OVR() iv IVT_ADDR_TIMER1_OVF ics ICS_AUTO {

     TOV1_bit = 0;                     // Clear Timer1 interrupt flag
    if(timer_cnt == 1){                 // When 0.5 second time elapsed
      freq = cnt;                       // Store result in freq
      cnt = 0;                          // Reset counter
      timer_cnt = 0;                    // Reset timer_cnt
      freq_read_done ++;
    }
    else
      timer_cnt++;                      // Increment timer_cnt if not elapsed 1 second

    TCNT1H = 0xC2;                       // First write higher byte to TMR1
    TCNT1L = 0xF7;                       // Write lower byte to TMR1
}


/**************************************************************************************************
* Function Meter()
* -------------------------------------------------------------------------------------------------
* Overview: Function displays linear interpretation on portD LEDS
* Input: frequency
* Output: Nothing
**************************************************************************************************/
char Meter(unsigned long f) {
  char temp = 0x00, i = 0x01;
  unsigned long level = 100;

  while (level <= 500) {
    if (freq >= level) temp = temp | i;
    else temp = temp & (~i);
    level = level + 50;
    i = i << 1;
  }
  return temp;
}

/**************************************************************************************************
* Main
**************************************************************************************************/
void main() {
  freq = 0;
  old_freq  = 0;
  cnt = 0;
  timer_cnt = 0;
  DDRD=0xFF;
  DIODES_direction = 0xFF;
  OUT_direction = 0;                    // INT0 pin direction selected

  DIODES = 0x00;

  UART1_Init(19200);
  Delay_ms(100);

// Timer1 works in 16-bit mode with internal instruction cycle clock and prescaler 1:8
// If clock is 8MHz and prescaler is set to 1/64, frequency of oscillator will be 125Khz
// Because 16-bit timer is not big enough to count from 0 to 125000 in one second, it will be necessary
// to count on smaller interval, 1/8 second. So 125000/8 = 15625 therefore we will set TMR1 register
// value on 65536 - 15625 = 49911 (0xC2F7)
// TMR1 overflow from 0xFFFF to 0x0000 will generate interrupt with 1/8 second period.

  TCNT1H = 0xC2;                        // First write higher byte to TMR1
  TCNT1L = 0xF7;                        // Write lower byte to TMR1

  CS12_bit = 0;                         // Timer1 settings:
  CS11_bit = 1;                         // Prescaler is set
  CS10_bit = 1;                         // to Fosc/64 = 125KHz

  TOIE1_bit = 1;                        // Enable Timer1 overflow interrupt

  ISC00_bit = 1;                        //Interrupt on PD2 is triggered
  ISC01_bit = 1;                        //on rising edge

  INT0_bit = 1;                         //Interrupt on PD2 is Enabled

  SREG_I_bit = 1;                       //Enable Interrupts
  while(1){

    if (freq_read_done == 2){

      SREG_I_bit = 0;                   // Disable interrupts
      freq_read_done = 0;
      freq = freq << 1;                 // Doubling number of captured edges because sampling period is 0.5s
      LongToStr(freq, txt);             // Convert freq (long integer number) to a string
      DIODES = Meter(freq);             // Displaying result on PORTD diodes
      UART1_Write_Text(txt);            // Sending result via UART
      UART1_Write('\n');

      old_freq  = freq;
      SREG_I_bit = 1;                   //Enable Interrupts
    }
  }
}

/**************************************************************************************************
* End of File
**************************************************************************************************/